import React, { useState, useCallback, useEffect, useRef } from 'react';
import { categories, filters } from './data/challenges';
import { CategoryCard } from './components/CategoryCard';
import { ChallengeCard } from './components/ChallengeCard';
import { ProgressBadge } from './components/ProgressBadge';
import { CompletedList } from './components/CompletedList';
import { UserProfileModal } from './components/UserProfileModal';
import { ConfirmationModal } from './components/ConfirmationModal';
import { SaveProgressInline } from './components/SaveProgressInline';
import { FilterTags } from './components/FilterTags';
import { FeaturesList } from './components/FeaturesList';
import { Footer } from './components/Footer';
import { LevelProgress } from './components/LevelProgress';
import { AchievementBadges } from './components/AchievementBadges';
import { LoadingSpinner } from './components/LoadingSpinner';
import { trackPageView } from './utils/analytics';
import { getCache, setCache, CACHE_KEYS, CACHE_DURATION } from './utils/cache';
import { 
  Sparkles, Award, UserCircle, LogOut, Trash2, Star, Trophy, 
  Target, Zap, Crown, Diamond, Flame, Medal, Rocket, Menu,
  X
} from 'lucide-react';
import { CompletedChallenge, UserProfile, Achievement } from './types';
import { v4 as uuidv4 } from 'uuid';
import { supabase } from './utils/supabase';
import { signIn, signUp, isSessionValid, updateUserChallenges, updateGuestChallenges } from './utils/auth';
import { logEvent, getClientInfo } from './utils/logging';

const DAILY_ACHIEVEMENTS: Achievement[] = [
  {
    threshold: 5,
    title: "High Five! 🖐️",
    message: "You're on fire! Five challenges completed - you're making incredible progress!",
    icon: Star,
    color: "from-yellow-400 to-orange-500"
  },
  {
    threshold: 10,
    title: "Perfect Ten! 🌟",
    message: "Double digits! Your dedication to self-improvement is truly inspiring!",
    icon: Trophy,
    color: "from-purple-500 to-pink-500"
  },
  {
    threshold: 15,
    title: "Unstoppable! 🚀",
    message: "Fifteen challenges down! You're absolutely crushing it today!",
    icon: Target,
    color: "from-blue-500 to-indigo-600"
  },
  {
    threshold: 20,
    title: "Challenge Master! 👑",
    message: "Twenty challenges in one day! You're setting new records!",
    icon: Crown,
    color: "from-emerald-500 to-teal-600"
  }
];

const STREAK_ACHIEVEMENTS: Achievement[] = [
  {
    threshold: 3,
    title: "Three Day Streak! ⚡",
    message: "You've completed challenges for 3 days in a row! Keep the momentum going!",
    icon: Zap,
    color: "from-amber-500 to-orange-600"
  },
  {
    threshold: 7,
    title: "Week Warrior! 🔥",
    message: "A full week of daily challenges! Your consistency is remarkable!",
    icon: Flame,
    color: "from-red-500 to-pink-600"
  },
  {
    threshold: 15,
    title: "Fortnight Champion! 🏅",
    message: "15 days of continuous improvement! You're building lasting habits!",
    icon: Medal,
    color: "from-violet-500 to-purple-600"
  },
  {
    threshold: 30,
    title: "Monthly Master! 💎",
    message: "30 days of dedication! You've achieved something truly extraordinary!",
    icon: Diamond,
    color: "from-cyan-500 to-blue-600"
  }
];

function App() {
  const [isMobileMenuOpen, setIsMobileMenuOpen] = useState(false);
  const [selectedFilters, setSelectedFilters] = useState<string[]>([]);
  const [completedChallenges, setCompletedChallenges] = useState<Set<string>>(new Set());
  const challengeRef = useRef<HTMLDivElement>(null);
  const [selectedCategory, setSelectedCategory] = useState<string | null>(null);
  const [currentChallenge, setCurrentChallenge] = useState<number>(0);
  const [isProfileModalOpen, setIsProfileModalOpen] = useState(false);
  const [isConfirmClearOpen, setIsConfirmClearOpen] = useState(false);
  const [showAchievement, setShowAchievement] = useState(false);
  const [currentAchievement, setCurrentAchievement] = useState<Achievement | null>(null);
  const [isSigningUp, setIsSigningUp] = useState(true);
  const [isLoading, setIsLoading] = useState(true);
  const [userProfile, setUserProfile] = useState<UserProfile | null>(null);
  const [usedChallenges, setUsedChallenges] = useState<Set<string>>(new Set());

  useEffect(() => {
    const initializeUserProfile = async () => {
      try {
        const cached = getCache(CACHE_KEYS.USER_PROFILE);
        if (cached) {
          setUserProfile(cached);
          setIsLoading(false);
          return;
        }

        const saved = localStorage.getItem('userProfile');
        const profile = saved ? JSON.parse(saved) : {
          username: 'Guest',
          completedChallenges: {},
          joinedAt: new Date().toISOString(),
          currentStreak: 0,
          longestStreak: 0,
          points: 0,
          level: 1,
          nextLevelPoints: 100
        };

        if (isSessionValid()) {
          const storedUser = localStorage.getItem('currentUser');
          if (storedUser) {
            const user = JSON.parse(storedUser);
            setUserProfile(user);
            setCache(CACHE_KEYS.USER_PROFILE, user, CACHE_DURATION.LONG);
          } else {
            setUserProfile(profile);
            setCache(CACHE_KEYS.USER_PROFILE, profile, CACHE_DURATION.LONG);
          }
        } else {
          setUserProfile(profile);
          setCache(CACHE_KEYS.USER_PROFILE, profile, CACHE_DURATION.LONG);
        }
      } catch (error) {
        console.error('Error initializing user profile:', error);
        const fallbackProfile = {
          username: 'Guest',
          completedChallenges: {},
          joinedAt: new Date().toISOString(),
          currentStreak: 0,
          longestStreak: 0,
          points: 0,
          level: 1,
          nextLevelPoints: 100
        };
        setUserProfile(fallbackProfile);
        setCache(CACHE_KEYS.USER_PROFILE, fallbackProfile, CACHE_DURATION.LONG);
      } finally {
        setTimeout(() => setIsLoading(false), 100);
      }
    };

    initializeUserProfile();
    trackPageView();
  }, []);

  const handleLogin = async (username: string, password: string) => {
    try {
      const { success, error, user } = await signIn(username, password);
      if (success && user) {
        setUserProfile(user);
        localStorage.setItem('currentUser', JSON.stringify(user));
        setIsProfileModalOpen(false);
      } else {
        throw new Error(error || 'Failed to sign in');
      }
    } catch (error) {
      console.error('Login error:', error);
      throw error;
    }
  };

  const handleSignUp = async (username: string, password: string) => {
    try {
      const { success, error } = await signUp(username, password);
      if (success) {
        const { success: signInSuccess, error: signInError, user } = await signIn(username, password);
        if (signInSuccess && user) {
          setUserProfile(user);
          localStorage.setItem('currentUser', JSON.stringify(user));
          setIsProfileModalOpen(false);
        } else {
          throw new Error(signInError || 'Failed to sign in after signup');
        }
      } else {
        throw new Error(error || 'Failed to create account');
      }
    } catch (error) {
      console.error('Signup error:', error);
      throw error;
    }
  };

  const handleLogout = async () => {
    try {
      const clientInfo = await getClientInfo();
      await logEvent({
        level: 'info',
        event: 'user.logout',
        details: { username: userProfile?.username },
        ...clientInfo
      });

      clearUserData();
      setSelectedCategory(null);
      setCurrentChallenge(0);
      setUsedChallenges(new Set());
    } catch (error) {
      console.error('Error during logout:', error);
    }
  };

  const clearUserData = () => {
    localStorage.removeItem('currentUser');
    localStorage.removeItem('sessionExpiry');
    localStorage.removeItem('userProfile');
    setUserProfile({
      username: 'Guest',
      completedChallenges: {},
      joinedAt: new Date().toISOString(),
      currentStreak: 0,
      longestStreak: 0,
      points: 0,
      level: 1,
      nextLevelPoints: 100
    });
  };

  const handleClearHistory = () => {
    const updatedProfile = {
      ...userProfile!,
      completedChallenges: {},
      currentStreak: 0,
      lastStreak: undefined,
      points: 0,
      level: 1,
      nextLevelPoints: 100
    };

    setUserProfile(updatedProfile);

    if (userProfile?.username !== 'Guest') {
      localStorage.setItem(`userProfile_${userProfile.username}`, JSON.stringify(updatedProfile));
    }
    localStorage.setItem('userProfile', JSON.stringify(updatedProfile));
  };

  const today = new Date().toISOString().split('T')[0];
  const completedToday = userProfile?.completedChallenges?.[today] || [];

  useEffect(() => {
    if (userProfile && userProfile.username !== 'Guest') {
      localStorage.setItem('userProfile', JSON.stringify(userProfile));
      localStorage.setItem(`userProfile_${userProfile.username}`, JSON.stringify(userProfile));
    }
  }, [userProfile]);

  useEffect(() => {
    const checkAchievements = () => {
      const count = completedToday.length;
      const dailyAchievement = DAILY_ACHIEVEMENTS.find(m => m.threshold === count);
      
      if (dailyAchievement) {
        setCurrentAchievement(dailyAchievement);
        setShowAchievement(true);
        setTimeout(() => setShowAchievement(false), 7000);
      }
    };

    checkAchievements();
  }, [completedToday.length]);

  useEffect(() => {
    const updateStreak = () => {
      if (!userProfile || !completedToday.length) return;

      const lastDate = userProfile.lastStreak;
      const yesterday = new Date();
      yesterday.setDate(yesterday.getDate() - 1);
      const yesterdayStr = yesterday.toISOString().split('T')[0];

      let newStreak = userProfile.currentStreak;
      
      if (lastDate === yesterdayStr) {
        newStreak += 1;
      } else if (lastDate !== today) {
        newStreak = 1;
      }

      const newLongestStreak = Math.max(newStreak, userProfile.longestStreak || 0);
      
      const streakAchievement = STREAK_ACHIEVEMENTS.find(m => m.threshold === newStreak);
      if (streakAchievement) {
        setTimeout(() => {
          setCurrentAchievement(streakAchievement);
          setShowAchievement(true);
          setTimeout(() => setShowAchievement(false), 7000);
        }, 1000);
      }

      setUserProfile(prev => ({
        ...prev!,
        lastStreak: today,
        currentStreak: newStreak,
        longestStreak: newLongestStreak
      }));
    };

    updateStreak();
  }, [completedToday.length, today, userProfile?.lastStreak]);

  const getRandomChallenge = useCallback((category: string) => {
    const challenges = categories[category].challenges;
    const todaysChallenges = userProfile?.completedChallenges?.[today] || [];
    const completedTitles = new Set(todaysChallenges.map(c => c.title));
    
    let availableChallenges = challenges.filter(challenge => {
      const matchesFilters = selectedFilters.length === 0 || 
        selectedFilters.every(filter => challenge.filters?.includes(filter));
      return matchesFilters && !completedTitles.has(challenge.title);
    });

    if (usedChallenges.size > 0) {
      availableChallenges = availableChallenges.filter(
        challenge => !usedChallenges.has(challenge.title)
      );
    }
    
    if (availableChallenges.length === 0) {
      if (usedChallenges.size > 0) {
        setUsedChallenges(new Set());
        getRandomChallenge(category);
        return;
      }
      alert("No more challenges available with the current filters. Try different filters or another category!");
      setSelectedCategory(null);
      return;
    }
    
    const randomIndex = Math.floor(Math.random() * availableChallenges.length);
    const selectedChallenge = availableChallenges[randomIndex];
    const challengeIndex = challenges.findIndex(c => c.title === selectedChallenge.title);
    
    setUsedChallenges(prev => {
      const updated = new Set(prev);
      updated.add(selectedChallenge.title);
      if (updated.size > 10) {
        const [firstItem] = updated;
        updated.delete(firstItem);
      }
      return updated;
    });
    
    setCurrentChallenge(challengeIndex);
  }, [userProfile, selectedFilters, usedChallenges]);

  const scrollToChallenge = () => {
    if (challengeRef.current) {
      challengeRef.current.scrollIntoView({ behavior: 'smooth', block: 'center' });
    }
  };

  const handleCategorySelect = (category: string) => {
    setSelectedCategory(category);
    getRandomChallenge(category);
    setTimeout(scrollToChallenge, 100);
  };

  const handleSurpriseMe = () => {
    const categoryKeys = Object.keys(categories);
    const randomCategory = categoryKeys[Math.floor(Math.random() * categoryKeys.length)];
    handleCategorySelect(randomCategory);
  };

  const handleShare = () => {
    if (!selectedCategory) return;
    
    const challenge = categories[selectedCategory].challenges[currentChallenge];
    const text = `I just completed this 5-minute challenge on Today's 5-Minute Fix: "${challenge.title}" - Try it yourself!`;
    
    if (navigator.share) {
      navigator.share({
        title: "Today's 5-Minute Fix",
        text: text,
        url: window.location.href,
      }).catch(console.error);
    } else {
      navigator.clipboard.writeText(text)
        .then(() => alert('Challenge copied to clipboard!'))
        .catch(console.error);
    }
  };

  const handleComplete = async () => {
    if (!userProfile || !selectedCategory) return;

    const challenge = categories[selectedCategory].challenges[currentChallenge];
    const today = new Date().toISOString().split('T')[0];
    
    const completedChallenge: CompletedChallenge = {
      id: uuidv4(),
      category: categories[selectedCategory].name,
      title: challenge.title,
      completedAt: new Date().toISOString()
    };

    const updatedProfile = {
      ...userProfile,
      completedChallenges: {
        ...userProfile.completedChallenges,
        [today]: [...(userProfile.completedChallenges[today] || []), completedChallenge]
      }
    };

    if (userProfile.username !== 'Guest') {
      const success = await updateUserChallenges(
        userProfile.username, 
        updatedProfile.completedChallenges
      );

      if (!success) {
        console.error('Failed to update challenges in database');
        return;
      }

      const { data: userData, error: userError } = await supabase
        .from('registered_users')
        .select('points, level, next_level_points')
        .eq('username', userProfile.username)
        .single();

      if (!userError && userData) {
        updatedProfile.points = userData.points;
        updatedProfile.level = userData.level;
        updatedProfile.nextLevelPoints = userData.next_level_points;
      }
    } else {
      const success = await updateGuestChallenges(updatedProfile.completedChallenges);
      
      if (!success) {
        console.error('Failed to update guest challenges');
        return;
      }

      updatedProfile.points = (updatedProfile.points || 0) + 10;
      while (updatedProfile.points >= updatedProfile.nextLevelPoints) {
        updatedProfile.level += 1;
        updatedProfile.nextLevelPoints = Math.floor(100 * Math.pow(1.25, updatedProfile.level - 1));
      }
    }

    setUserProfile(updatedProfile);
    localStorage.setItem('userProfile', JSON.stringify(updatedProfile));
    
    if (userProfile.username !== 'Guest') {
      localStorage.setItem(`userProfile_${userProfile.username}`, JSON.stringify(updatedProfile));
    }
  };

  const handleHeaderSignIn = () => {
    setIsProfileModalOpen(true);
    setIsSigningUp(false);
  };

  const handleSaveProgress = () => {
    setIsProfileModalOpen(true);
    setIsSigningUp(true);
  };

  const shouldShowSaveProgress = userProfile?.username === 'Guest' && completedToday.length > 0;

  if (isLoading || !userProfile) {
    return (
      <div className="min-h-screen bg-gray-50 flex items-center justify-center">
        <LoadingSpinner />
      </div>
    );
  }

  return (
    <div className="min-h-screen bg-gray-50 flex flex-col">
      <div className="bg-white shadow-sm sticky top-0 z-50">
        <div className="max-w-6xl mx-auto px-4 py-4">
          <div className="flex justify-between items-center">
            <div className="flex items-center space-x-2">
              <Sparkles className="w-6 h-6 text-blue-500" />
              <h1 className="text-xl font-bold text-gray-900">5-Minute Task</h1>
            </div>
            
            <div className="hidden md:flex items-center space-x-4">
              <ProgressBadge count={completedToday.length} />
              {userProfile.currentStreak > 0 && (
                <div className="flex items-center space-x-2 bg-gradient-to-r from-orange-500 to-red-500 text-white px-4 py-2 rounded-full">
                  <Flame className="w-5 h-5" />
                  <span className="font-medium">{userProfile.currentStreak} day streak!</span>
                </div>
              )}
              {userProfile.username !== 'Guest' ? (
                <div className="flex items-center space-x-4">
                  <span className="text-gray-700">
                    <UserCircle className="w-5 h-5 inline mr-2" />
                    {userProfile.username}
                  </span>
                  <button
                    onClick={handleLogout}
                    className="inline-flex items-center space-x-2 px-4 py-2 rounded-lg bg-red-100 hover:bg-red-200 text-red-700 transition-colors"
                  >
                    <LogOut className="w-4 h-4" />
                    <span>Logout</span>
                  </button>
                </div>
              ) : (
                <button
                  onClick={handleHeaderSignIn}
                  className="inline-flex items-center justify-center space-x-2 px-4 py-2 rounded-lg bg-blue-500 hover:bg-blue-600 text-white transition-colors"
                >
                  <UserCircle className="w-5 h-5" />
                  <span>Sign In</span>
                </button>
              )}
            </div>

            <button
              onClick={() => setIsMobileMenuOpen(!isMobileMenuOpen)}
              className="md:hidden p-2 rounded-lg hover:bg-gray-100"
            >
              {isMobileMenuOpen ? (
                <X className="w-6 h-6 text-gray-600" />
              ) : (
                <Menu className="w-6 h-6 text-gray-600" />
              )}
            </button>
          </div>
        </div>

        {isMobileMenuOpen && (
          <div className="md:hidden border-t border-gray-100">
            <div className="max-w-6xl mx-auto px-4 py-4 space-y-4">
              <ProgressBadge count={completedToday.length} />
              {userProfile.currentStreak > 0 && (
                <div className="flex items-center space-x-2 bg-gradient-to-r from-orange-500 to-red-500 text-white px-4 py-2 rounded-full">
                  <Flame className="w-5 h-5" />
                  <span className="font-medium">{userProfile.currentStreak} day streak!</span>
                </div>
              )}
              {userProfile.username !== 'Guest' ? (
                <div className="space-y-2">
                  <div className="flex items-center space-x-2 text-gray-700">
                    <UserCircle className="w-5 h-5" />
                    <span>{userProfile.username}</span>
                  </div>
                  <button
                    onClick={handleLogout}
                    className="w-full inline-flex items-center justify-center space-x-2 px-4 py-2 rounded-lg bg-red-100 hover:bg-red-200 text-red-700 transition-colors"
                  >
                    <LogOut className="w-4 h-4" />
                    <span>Logout</span>
                  </button>
                </div>
              ) : (
                <button
                  onClick={handleHeaderSignIn}
                  className="w-full inline-flex items-center justify-center space-x-2 px-4 py-2 rounded-lg bg-blue-500 hover:bg-blue-600 text-white transition-colors"
                >
                  <UserCircle className="w-5 h-5" />
                  <span>Sign In</span>
                </button>
              )}
            </div>
          </div>
        )}
      </div>

      <main className="max-w-6xl mx-auto px-4 py-6 sm:py-12 flex-grow">
        <div className="text-center mb-8">
          <h2 className="text-3xl sm:text-4xl font-bold text-gray-900 mb-4 font-display">
            Transform Your Day with 5-Minute Tasks
          </h2>
          <p className="text-lg sm:text-xl text-gray-600 max-w-2xl mx-auto font-body">
            Small actions, big impact. Choose a quick task and start your journey to self-improvement today.
          </p>
        </div>

        <FeaturesList />

        <FilterTags
          filters={filters}
          selectedFilters={selectedFilters}
          onFilterChange={setSelectedFilters}
        />

        <div className="grid grid-cols-1 sm:grid-cols-2 lg:grid-cols-3 gap-4 sm:gap-6 mb-8">
          {Object.entries(categories).map(([key, category]) => (
            <CategoryCard
              key={key}
              name={category.name}
              description={category.description}
              color={category.color}
              onClick={() => handleCategorySelect(key)}
              selected={selectedCategory === key}
            />
          ))}
        </div>

        <div className="text-center mb-8 sm:mb-12">
          <button
            onClick={handleSurpriseMe}
            className="inline-flex items-center px-6 py-3 rounded-lg bg-gradient-to-r from-purple-500 to-pink-500 text-white font-medium hover:opacity-90 transition-opacity duration-200"
          >
            <Sparkles className="w-5 h-5 mr-2" />
            Surprise Me!
          </button>
        </div>

        {selectedCategory && (
          <div ref={challengeRef} className="mt-8">
            <ChallengeCard
              {...categories[selectedCategory].challenges[currentChallenge]}
              color={categories[selectedCategory].color}
              onTryAnother={() => getRandomChallenge(selectedCategory)}
              onShare={handleShare}
              onComplete={handleComplete}
            />
          </div>
        )}

        {shouldShowSaveProgress && (
          <SaveProgressInline
            onSave={handleSaveProgress}
            completedCount={completedToday.length}
          />
        )}

        {completedToday.length > 0 && (
          <div className="mt-12">
            <div className="text-center mb-8">
              <div className="inline-flex items-center space-x-2 bg-gradient-to-r from-yellow-500 to-amber-500 text-white px-5 py-2 rounded-full shadow-md">
                <Award className="w-5 h-5" />
                <span className="text-sm font-medium">
                  {completedToday.length === 1 
                    ? "You've completed your first challenge today! Keep going! 🎉" 
                    : `Amazing progress! You've completed ${completedToday.length} challenges today! 🌟`}
                </span>
              </div>
            </div>

            {userProfile && (
              <>
                <div className="mb-8">
                  <LevelProgress
                    level={userProfile.level}
                    points={userProfile.points}
                    nextLevelPoints={userProfile.nextLevelPoints}
                  />
                </div>
                <div className="mb-8">
                  <AchievementBadges
                    completedCount={Object.values(userProfile.completedChallenges).reduce(
                      (total, challenges) => total + challenges.length,
                      0
                    )}
                    currentStreak={userProfile.currentStreak}
                    longestStreak={userProfile.longestStreak}
                    level={userProfile.level}
                  />
                </div>
              </>
            )}
            
            <div className="flex justify-between items-center mb-6">
              <h2 className="text-2xl font-bold">Your Completed Challenges</h2>
              <button
                onClick={() => setIsConfirmClearOpen(true)}
                className="inline-flex items-center space-x-2 px-4 py-2 rounded-lg bg-yellow-100 hover:bg-yellow-200 text-yellow-700 transition-colors"
              >
                <Trash2 className="w-4 h-4" />
                <span>Clear History</span>
              </button>
            </div>
            <CompletedList
              challenges={completedToday}
              date="Today"
            />
          </div>
        )}
      </main>

      <Footer />

      <UserProfileModal
        isOpen={isProfileModalOpen}
        onClose={() => setIsProfileModalOpen(false)}
        onLogin={handleLogin}
        onSignUp={handleSignUp}
        initialMode={isSigningUp ? 'signup' : 'signin'}
      />

      <ConfirmationModal
        isOpen={isConfirmClearOpen}
        onClose={() => setIsConfirmClearOpen(false)}
        onConfirm={handleClearHistory}
        title="Clear Challenge History"
        message="Are you sure you want to clear your challenge history? This action cannot be undone and all your completed challenges will be permanently removed."
      />
    </div>
  );
}

export default App;